import {reactive} from "vue"

export type BreakpointKey = 'sm' | 'md' | 'lg' | 'xl'
export type ScreenSize = 'XS' | 'SM' | 'MD' | 'LG' | 'XL'
export const ScreenSizeArray = ['XS', 'SM', 'MD', 'LG', 'XL']

/**
 * 반응형 디자인을 위한 breakpoints.
 *
 * variables.scss 의 값과 일치해야 한다.
 */
export const BREAKPOINTS: Record<BreakpointKey, string> = {
  sm: import.meta.env.VITE_BREAKPOINT_SM || '576',
  md: import.meta.env.VITE_BREAKPOINT_MD || '768',
  lg: import.meta.env.VITE_BREAKPOINT_LG || '992',
  xl: import.meta.env.VITE_BREAKPOINT_XL || '1140',
}

export const MEDIA_QUERIES: Record<BreakpointKey, string> = {
  sm: `(min-width: ${BREAKPOINTS.sm}px)`,
  md: `(min-width: ${BREAKPOINTS.md}px)`,
  lg: `(min-width: ${BREAKPOINTS.lg}px)`,
  xl: `(min-width: ${BREAKPOINTS.xl}px)`
}

export type ResponsiveConfig = {
  screenSize: ScreenSize
}

const globalResponsiveConfig: ResponsiveConfig = reactive<ResponsiveConfig>({
  screenSize: 'XS'
})

const refreshScreenSize = () => {
  const width = window.innerWidth
  globalResponsiveConfig.screenSize =
      (width >= Number(BREAKPOINTS.xl)) ? 'XL' :
          (width >= Number(BREAKPOINTS.lg)) ? 'LG' :
              (width >= Number(BREAKPOINTS.md)) ? 'MD' :
                  (width >= Number(BREAKPOINTS.sm)) ? 'SM' : 'XS'
}
refreshScreenSize()

window?.addEventListener('resize', refreshScreenSize, {passive: true})

export const responsiveUtil = {

  currentScreenSize(): ScreenSize {
    return globalResponsiveConfig.screenSize
  },

  isScreenSizeSmallerThan(size: ScreenSize): boolean {
    return ScreenSizeArray.indexOf(globalResponsiveConfig.screenSize) < ScreenSizeArray.indexOf(size)
  },

  isScreenSizeEqualOrSmallerThan(size: ScreenSize): boolean {
    return ScreenSizeArray.indexOf(globalResponsiveConfig.screenSize) <= ScreenSizeArray.indexOf(size)
  },

  isScreenSizeEqualOrGreaterThan(size: ScreenSize): boolean {
    return ScreenSizeArray.indexOf(globalResponsiveConfig.screenSize) >= ScreenSizeArray.indexOf(size)
  },

  isScreenSizeGreaterThan(size: ScreenSize): boolean {
    return ScreenSizeArray.indexOf(globalResponsiveConfig.screenSize) > ScreenSizeArray.indexOf(size)
  }
}